@@ -3,34 +3,40 @@ package Config::JSON;
 use warnings;
 use strict;
 use Carp;
+use Class::InsideOut qw(readonly id register private);
 use JSON;
 use List::Util;
+use version; our $VERSION = qv('1.1.0');
 
-use version; our $VERSION = qw('1.0.3');
+
+use constant FILE_HEADER    => "# config-file-type: JSON 1\n";
+use constant JSON_OPTIONS   => {pretty => 1, indent => 4, autoconv=>0, skipinvalid=>1};
+
+
+readonly    getFilePath     => my %filePath;    # path to config file
+private     config          => my %config;      # in memory config file
 
 #-------------------------------------------------------------------
 sub addToArray {
-        my $self = shift;
-        my $property = shift;
-        my $value = shift;
-        my $array = $self->get($property);
-        unless (defined List::Util::first { $value eq $_ } @{$array}) { # check if it already exists
+    my $self = shift;
+    my $property = shift;
+    my $value = shift;
+    my $array = $self->get($property);
+    unless (defined List::Util::first { $value eq $_ } @{$array}) { # check if it already exists
 		# add it
-        	push(@{$array}, $value);
-        	$self->set($property, $array);
+      	push(@{$array}, $value);
+      	$self->set($property, $array);
 	}
 }
 
 
 #-------------------------------------------------------------------
 sub addToHash {
-        my $self = shift;
-        my $property = shift;
-        my $key = shift;
-        my $value = shift;
-        my $hash = $self->get($property);
-        $hash->{$key} = $value;
-        $self->set($property, $hash);
+    my $self = shift;
+    my $property = shift;
+    my $key = shift;
+    my $value = shift;
+    $self->set($property."/".$key, $value);
 }
 
 
@@ -38,12 +44,13 @@ sub addToHash {
 sub create {
 	my $class = shift;
 	my $filename = shift;
-        if (open(my $FILE,">",$filename)) {
-                print $FILE "# config-file-type: JSON 1\n{ }\n";
-                close($FILE);
-        } else {
-                carp "Can't write to config file ".$filename;
-        }
+    if (open(my $FILE,">",$filename)) {
+        print $FILE FILE_HEADER."\n{ }\n";
+        close($FILE);
+    } 
+    else {
+        carp "Can't write to config file ".$filename;
+    }
 	return $class->new($filename);	
 }
 
@@ -51,102 +58,116 @@ sub create {
 
 #-------------------------------------------------------------------
 sub delete {
-        my $self = shift;
-        my $param = shift;
-        delete $self->{_config}{$param};
-        if (open(my $FILE,">",$self->getFilename)) {
-                print $FILE "# config-file-type: JSON 1\n".objToJson($self->{_config}, {pretty => 1, indent => 4, autoconv=>0, skipinvalid=>1});
-                close($FILE);
-        } else {
-                carp "Can't write to config file ".$self->getFilename;
-        }
+    my $self = shift;
+    my $param = shift;
+    my $directive   = $config{id $self};
+    my @parts       = split "/", $param;
+    my $lastPart    = pop @parts;
+    foreach my $part (@parts) {
+        $directive = $directive->{$part};
+    }
+    delete $directive->{$lastPart};
+    if (open(my $FILE,">",$self->getFilePath)) {
+        print $FILE FILE_HEADER."\n".objToJson($config{id $self}, JSON_OPTIONS);
+        close($FILE);
+    } 
+    else {
+        carp "Can't write to config file ".$self->getFilePath;
+    }
 }
 
 #-------------------------------------------------------------------
 sub deleteFromArray {
-        my $self = shift;
-        my $property = shift;
-        my $value = shift;
-        my $array = $self->get($property);
-        for (my $i = 0; $i < scalar(@{$array}); $i++) {
-                if ($array->[$i] eq $value) {
-                        splice(@{$array}, $i, 1);
-                        last;
-                }
+    my $self = shift;
+    my $property = shift;
+    my $value = shift;
+    my $array = $self->get($property);
+    for (my $i = 0; $i < scalar(@{$array}); $i++) {
+        if ($array->[$i] eq $value) {
+            splice(@{$array}, $i, 1);
+            last;
         }
-        $self->set($property, $array);
+    }
+    $self->set($property, $array);
 }
 
 
 #-------------------------------------------------------------------
 sub deleteFromHash {
-        my $self = shift;
-        my $property = shift;
-        my $key = shift;
-        my $hash = $self->get($property);
-        delete $hash->{$key};
-        $self->set($property, $hash);
-}
-
-
-#-------------------------------------------------------------------
-sub DESTROY {
-        my $self = shift;
-        undef $self;
+    my $self = shift;
+    my $property = shift;
+    my $key = shift;
+    $self->delete($property."/".$key);
 }
 
 
 #-------------------------------------------------------------------
 sub get {
-        my $self = shift;
-        my $param = shift;
-        return $self->{_config}{$param};
+    my $self        = shift;
+    my $property    = shift;
+    my $value       = $config{id $self};
+    foreach my $part (split "/", $property) {
+        $value = $value->{$part};
+    }
+    return $value;
 }
 
 
 #-------------------------------------------------------------------
 sub getFilename {
-        my $self = shift;
-        return $self->{_pathToFile};
+    my $self = shift;
+    my @path = split "/", $self->getFilePath;
+    return pop @path;
 }
 
 
 #-------------------------------------------------------------------
 sub new {
-        my $class = shift;
-        my $pathToFile = shift;
-        my $json = "";
-        if (open(my $FILE, "<", $pathToFile)) {
-                while (my $line = <$FILE>) {
-                        $json .= $line unless ($line =~ /^\s*#/);
-                }
-                close($FILE);
-                my $conf = jsonToObj($json);
-                croak "Couldn't parse JSON in config file '$pathToFile'\n" unless ref $conf;
-                my $self = {_pathToFile=>$pathToFile, _config=>$conf};
-                bless $self, $class;
-                return $self;
-        } else {
-                croak "Cannot read config file: ".$pathToFile;
+    my $class = shift;
+    my $pathToFile = shift;
+    my $json = "";
+    if (open(my $FILE, "<", $pathToFile)) {
+        while (my $line = <$FILE>) {
+            $json .= $line unless ($line =~ /^\s*#/);
         }
+        close($FILE);
+        my $conf = jsonToObj($json);
+        croak "Couldn't parse JSON in config file '$pathToFile'\n" unless ref $conf;
+        my $self = register($class);
+        $filePath{id $self} = $pathToFile;
+        $config{id $self}   = $conf;
+        return $self;
+    } 
+    else {
+        croak "Cannot read config file: ".$pathToFile;
+    }
 }
 
 
 #-------------------------------------------------------------------
 sub set {
-        my $self = shift;
-        my $param = shift;
-        my $value = shift;
-        $self->{_config}{$param} = $value;
-        if (open(my $FILE, ">" ,$self->getFilename)) {
-                print $FILE "# config-file-type: JSON 1\n".objToJson($self->{_config}, {pretty => 1, indent => 4, autoconv=>0, skipinvalid=>1});
-                close($FILE);
-        } else {
-                carp "Can't write to config file ".$self->getFilename;
+    my $self        = shift;
+    my $property    = shift;
+    my $value       = shift;
+    my $directive   = $config{id $self};
+    my @parts       = split "/", $property;
+    my $lastPart    = pop @parts;
+    foreach my $part (@parts) {
+        unless (exists $directive->{$part}) {
+            $directive->{$part} = {};
         }
+        $directive = $directive->{$part};
+    }
+    $directive->{$lastPart} = $value;
+    if (open(my $FILE, ">" ,$self->getFilePath)) {
+        print {$FILE} FILE_HEADER."\n".objToJson($config{id $self}, JSON_OPTIONS);
+        close($FILE);
+    } 
+    else {
+        carp "Can't write to config file ".$self->getFilePath;
+    }
 }
 
-
 1; # Magic true value required at end of module
 __END__
 
@@ -157,7 +178,7 @@ Config::JSON - A JSON based config file system.
 
 =head1 VERSION
 
-This document describes Config::JSON version 1.0.0
+This document describes Config::JSON version 1.1.0
 
 
 =head1 SYNOPSIS
@@ -167,18 +188,19 @@ This document describes Config::JSON version 1.0.0
  my $config = Config::JSON->create($pathToFile);
  my $config = Config::JSON->new($pathToFile);
 
- my $element = $config->get($param);
+ my $element = $config->get($directive);
 
- $config->set($param,$value);
+ $config->set($directive,$value);
 
- $config->delete($param);
+ $config->delete($directive);
  $config->deleteFromHash($name, $key);
  $config->deleteFromArray($name, $value);
 
  $config->addToHash($name, $key, $value);
  $config->addToArray($name, $value);
 
- my $path = $config->getFilename;
+ my $path = $config->getFilePath;
+ my $filename = $config->getFilename;
 
 =head2 Example Config File
 
@@ -199,8 +221,6 @@ This document describes Config::JSON version 1.0.0
         }
  } 
 
-Note that you can put comments in the config file as long as # is the first non-space character on the line. However, if you use this API to write to the config file, your comments will be eliminated.
-
 
 =head1 DESCRIPTION
 
@@ -212,14 +232,28 @@ If you want to see it in action, it is used as the config file system in WebGUI
 
 Why build yet another config file system? Well there are a number of reasons: We used to use other config file parsers, but we kept running into limitations. We already use JSON in our app, so using JSON to store config files means using less memory because we already have the JSON parser in memory. In addition, with JSON we can have any number of hierarchcal data structures represented in the config file, whereas most config files will give you only one level of hierarchy, if any at all. JSON parses faster than XML and YAML. JSON is easier to read and edit than XML. Many other config file systems allow you to read a config file, but they don't provide any mechanism or utilities to write back to it. JSON is taint safe. JSON is easily parsed by languages other than Perl when we need to do that.
 
+=head2 Multi-level Directives
+
+You may of course access a directive called "foo", but since the config is basically a hash you can traverse
+multiple elements of the hash when specifying a directive name by simply delimiting each level with a slash, like
+"foo/bar". For example you may:
+
+ my $vitality = $config->get("stats/vitality");
+ $config->set("stats/vitality", 15);
+
+You may do this wherever you specify a directive name.
+
+=head2 Comments
+
+You can put comments in the config file as long as # is the first non-space character on the line. However, if you use this API to write to the config file, your comments will be eliminated.
 
 =head1 INTERFACE 
 
-=head2 addToArray ( property, value )
+=head2 addToArray ( directive, value )
 
-Adds a value to an array property in the config file.
+Adds a value to an array directive in the config file.
 
-=head3 property
+=head3 directive
 
 The name of the array.
 
@@ -228,11 +262,12 @@ The name of the array.
 The value to add.
 
 
-=head2 addToHash ( property, key, value )
+=head2 addToHash ( directive, key, value )
 
-Adds a value to a hash property in the config file.
+Adds a value to a hash directive in the config file. B<NOTE:> This is really the same as
+$config->set("directive/key", $value);
 
-=head3 property
+=head3 directive
 
 The name of the hash.
 
@@ -255,20 +290,20 @@ The path and filename of the file to create.
 
 
 
-=head2 delete ( param ) 
+=head2 delete ( directive ) 
 
 Deletes a key from the config file.
 
-=head3 param
+=head3 directive
 
-The name of the parameter to delete.
+The name of the directive to delete.
 
 
-=head2 deleteFromArray ( property, value )
+=head2 deleteFromArray ( directive, value )
 
-Deletes a value from an array property in the config file.
+Deletes a value from an array directive in the config file.
 
-=head3 property
+=head3 directive
 
 The name of the array.
 
@@ -278,11 +313,12 @@ The value to delete.
 
 
 
-=head2 deleteFromHash ( property, key )
+=head2 deleteFromHash ( directive, key )
 
-Delete a key from a hash property in the config file.
+Delete a key from a hash directive in the config file. B<NOTE:> This is really just the same as doing
+$config->delete("directive/key");
 
-=head3 property
+=head3 directive
 
 The name of the hash.
 
@@ -290,29 +326,26 @@ The name of the hash.
 
 The key to delete.
 
-=cut
 
 
+=head2 get ( directive ) 
 
+Returns the value of a particular directive from the config file.
 
-=head2 DESTROY ( )
+=head3 directive
 
-Deconstructor.
-
-
-
-=head2 get ( param ) 
-
-Returns the value of a particular parameter from the config file.
-
-=head3 param
-
-The name of the parameter to return.
+The name of the directive to return.
 
 
 
 =head2 getFilename ( )
 
+Returns the filename for this config.
+
+
+
+=head2 getFilePath ( ) 
+
 Returns the filename and path for this config.
 
 
@@ -327,13 +360,13 @@ A string representing a path such as "/etc/my-cool-config.conf".
 
 
 
-=head2 set ( param, value ) 
+=head2 set ( directive, value ) 
 
-Creates a new or updates an existing parameter in the config file.
+Creates a new or updates an existing directive in the config file.
 
-=head3 param
+=head3 directive
 
-A parameter name.
+A directive name.
 
 =head3 value
 
@@ -375,6 +408,8 @@ Config::JSON requires no configuration files or environment variables.
 
 =item List::Util
 
+=item Class::InsideOut
+
 =item Test::More
 
 =item version
